package io.gisbi.application.appcode.controller;

import io.gisbi.application.appcode.doman.TableData;
import io.gisbi.application.appcode.service.DatabaseService;
import io.gisbi.application.appcode.utils.ZipUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

@RestController
@RequestMapping("/project")
public class ProjectExportController {

    // 模板 ZIP 文件路径
    private static final String FIXED_ZIP_PATH = "E:/opt/gisbi2.0/";

    private static final String FIXED_ZIP_NAME = "a.zip";

    private static final String FIXED_ZIP_FULL_PATH= FIXED_ZIP_PATH + FIXED_ZIP_NAME;



    // 自定义数据库服务类
    @Autowired
    private DatabaseService databaseService;

    /**
     * 接口：上传项目 ZIP 包，解压 → 导出数据库部分数据 → 插入 SQL 文件 → 重新打包并下载
     */
    @PostMapping("/export")
    public void exportProjectWithDbData(
            @RequestParam("id") String id) throws Exception {

        String fixedZipPath = FIXED_ZIP_FULL_PATH;
        File fixedZipFile = new File(fixedZipPath);
        if (!fixedZipFile.exists()) {
            throw new FileNotFoundException("指定的 ZIP 文件不存在: " + fixedZipPath);
        }

        // 创建时间戳和临时目录路径
        String timestamp = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss"));
        Path tempExtractDir = Paths.get(FIXED_ZIP_PATH, "temp", "project-" + timestamp);

        try {
            Files.createDirectories(tempExtractDir);

            // 1. 解压模板 ZIP 到临时目录
            File uploadedZip = new File(tempExtractDir + ".zip");
            FileCopyUtils.copy(new FileInputStream(fixedZipFile), new FileOutputStream(uploadedZip));
            ZipUtils.unzip(uploadedZip, tempExtractDir.toFile());

            // 2. 获取解压后的根目录（自动识别第一个子目录）
            File[] extractedRoots = tempExtractDir.toFile().listFiles(File::isDirectory);
            if (extractedRoots == null || extractedRoots.length == 0) {
                throw new IOException("ZIP 解压后未找到项目根目录");
            }
            File projectRoot = extractedRoots[0];

            // 3. 获取 db 目录
            File dbDir = new File(projectRoot, "db");
            if (!dbDir.exists()) {
                boolean created = dbDir.mkdirs();
                if (!created) {
                    throw new IOException("无法创建 db 目录: " + dbDir.getAbsolutePath());
                }
            }

            // 4. 查询数据库并生成 SQL 文件，保存到 db 目录中
            generateSqlFile(id, new File(dbDir, "data.sql").getAbsolutePath());

            // 5. 重新打包整个项目根目录为新的 ZIP 文件
            String outputZipName = FIXED_ZIP_NAME.replaceFirst("\\..*$", "") +
                    "-with-sql-" + timestamp + ".zip";
            File finalZip = new File(FIXED_ZIP_PATH + "export/", outputZipName);
            ZipUtils.zipFolder(projectRoot, finalZip);
        }catch (Exception e){
            e.printStackTrace();
        }finally {
            // 7. 清理临时文件（可选）
            deleteDirectory(tempExtractDir.toFile());
        }
    }


    /**
     * 根据 id 查询数据库，并生成 SQL 文件（CREATE TABLE + INSERT INTO）
     */
    private File generateSqlFile(String id, String outputPath) throws IOException {
        // 自定义方法获取数据
        List<TableData> tableDataList = databaseService.getDataById(id);

        try (BufferedWriter writer = new BufferedWriter(new FileWriter(outputPath))) {
            for (TableData data : tableDataList) {
                // 写入建表语句
                writer.write(data.getTableCreateSql());
                writer.newLine();

                // 写入插入语句
                for (String insertSql : data.getInsertSqls()) {
                    writer.write(insertSql);
                    writer.newLine();
                }
            }
        }
        //        return null;
        return new File(outputPath);
    }

    private void deleteDirectory(File file) {
        if (file.isDirectory()) {
            File[] files = file.listFiles();
            if (files != null) {
                for (File f : files) {
                    deleteDirectory(f);
                }
            }
        }
        file.delete();
    }
}
